Imports System.Windows.Forms
Imports System.Drawing

Public Enum AnimationStage
    fadingIn
    stayingPut
    fadingOut
End Enum

Public Class MessageDisplayer
    Inherits System.Windows.Forms.Control

    Const ANIMATION_STEPS As Integer = 10
    Dim currentStage As AnimationStage _
        = AnimationStage.fadingIn
    Dim m_Messages As String() _
        = {"No Messages", "No Really"}
    Dim m_currentMessage As Integer = 0
    Dim m_previousMessage As Integer = 0
    Dim m_interval As Integer = 1
    Dim m_currentStep As Integer = 0
    Dim WithEvents m_timer As Timer


    Public Property Messages() As String()
        Get
            Return m_Messages
        End Get
        Set(ByVal Value As String())
            m_Messages = Value
            Me.Invalidate()
        End Set
    End Property

    Public Property currentMessage() As Integer
        Get
            Return m_currentMessage
        End Get
        Set(ByVal Value As Integer)
            If Value >= 0 AndAlso _
                 Value < m_Messages.Length Then
                m_currentMessage = Value
                Me.Invalidate()
            End If
        End Set
    End Property

    Public Property interval() As Integer
        Get
            Return m_interval
        End Get
        Set(ByVal Value As Integer)
            m_interval = Value
            If Not m_timer Is Nothing Then
                m_timer.Interval = _
                    (Value * 1000)
            End If
        End Set
    End Property


    Protected Overrides Sub OnEnabledChanged(ByVal e As System.EventArgs)
        SetupTimer()
    End Sub


    Private Sub SetupTimer()
        If Me.Enabled Then
            If m_timer Is Nothing Then
                m_timer = New System.Windows.Forms.Timer
            End If
            m_timer.Interval = (Me.interval * 1000)
            m_timer.Enabled = True
        Else
            If Not m_timer Is Nothing Then
                m_timer.Enabled = False
            End If
        End If
    End Sub
    Protected Overrides Sub OnPaint( _
            ByVal e As PaintEventArgs)

        Dim alpha As Integer
        Select Case Me.currentStage

            Case AnimationStage.stayingPut
                e.Graphics.Clear(Me.BackColor)
                alpha = 255

            Case AnimationStage.fadingIn
                e.Graphics.Clear(Me.BackColor)
                alpha = (256 \ ANIMATION_STEPS) _
                            * Me.m_currentStep

            Case AnimationStage.fadingOut
                e.Graphics.Clear(Me.BackColor)
                alpha = 255 - ((256 \ ANIMATION_STEPS) _
                            * Me.m_currentStep)

        End Select
        Debug.WriteLine(alpha)

        Dim fColor As Color _
            = Color.FromArgb(alpha, Me.ForeColor)
        Dim message As String = Me.m_Messages( _
            m_currentMessage)
        Dim layoutRect As Drawing.RectangleF
        layoutRect = New Drawing.RectangleF( _
            0, 0, Me.Width, Me.Height)
        layoutRect.Inflate(-10, -10)
        Dim messageBrush As Brush _
            = New SolidBrush(fColor)

        Dim myFormat As New _
            StringFormat(StringFormatFlags.LineLimit)
        myFormat.Trimming = _
            StringTrimming.EllipsisWord
        myFormat.Alignment = _
            StringAlignment.Center

        e.Graphics.SmoothingMode = _
            Drawing2D.SmoothingMode.AntiAlias
        e.Graphics.DrawString( _
            message, Me.Font, _
            messageBrush, layoutRect, myFormat)
    End Sub

    Private Sub m_timer_Tick( _
            ByVal sender As Object, _
            ByVal e As System.EventArgs) _
            Handles m_timer.Tick
        If Me.m_currentStep < Me.ANIMATION_STEPS Then
            Me.m_currentStep += 1
            Me.Invalidate()
        Else
            Me.m_currentStep = 0
            Select Case Me.currentStage
                Case AnimationStage.fadingIn
                    Me.currentStage = AnimationStage.stayingPut

                Case AnimationStage.fadingOut
                    If Me.m_currentMessage < _
                            Me.m_Messages.Length - 1 Then
                        Me.m_currentMessage += 1
                    Else
                        Me.m_currentMessage = 0
                    End If
                    Me.currentStage = AnimationStage.fadingIn

                Case AnimationStage.stayingPut
                    Me.currentStage = AnimationStage.fadingOut
            End Select
            Me.Invalidate()
        End If
    End Sub

    Public Sub New()
        Me.SetStyle(ControlStyles.AllPaintingInWmPaint, True)
        Me.SetStyle(ControlStyles.DoubleBuffer, True)
        Me.SetStyle(ControlStyles.ResizeRedraw, True)
        Me.SetStyle(ControlStyles.UserPaint, True)
        SetupTimer()
    End Sub
End Class
