Imports System.Net


Public Class WatchedURLEventArgs
    Inherits System.EventArgs
    Dim m_URLBeingWatched As String
    Dim m_Status As Integer
    Dim m_StatusDescription As String
    Dim m_CheckedTime As DateTime
    Dim m_URLAvailable As Boolean

    Public Property URLAvailable() As Boolean
        Get
            Return m_URLAvailable
        End Get
        Set(ByVal Value As Boolean)
            m_URLAvailable = Value
        End Set
    End Property

    Public Property URLBeingWatched() As String
        Get
            Return m_URLBeingWatched
        End Get
        Set(ByVal Value As String)
            m_URLBeingWatched = Value
        End Set
    End Property

    Public Property Status() As Integer
        Get
            Return m_Status
        End Get
        Set(ByVal Value As Integer)
            m_Status = Value
        End Set
    End Property

    Public Property StatusDescription() As String
        Get
            Return m_StatusDescription
        End Get
        Set(ByVal Value As String)
            m_StatusDescription = Value
        End Set
    End Property

    Public Property CheckedTime() As DateTime
        Get
            Return m_CheckedTime
        End Get
        Set(ByVal Value As DateTime)
            m_CheckedTime = Value
        End Set
    End Property
End Class

Public Class WebUtilities
    Inherits System.ComponentModel.Component

#Region "Internal Variables"

    Dim m_WatchedURL As String
    Dim m_SecondsBetweenCheck As Integer

    Dim m_lastCheck As DateTime
    Dim m_lastResponse As String
    Dim m_WatchedURLAvailable As Boolean

    Dim WithEvents m_internalTimer _
        As New System.Timers.Timer(1000)


#End Region

    Public Sub New()
        Me.m_internalTimer.AutoReset = True
        Me.m_internalTimer.Enabled = True
        Me.m_internalTimer.SynchronizingObject = Me.Container
    End Sub

    <WebPermission( _
        Security.Permissions.SecurityAction.Demand)> _
    Public Function RetrieveHTML(ByVal URL As String) As String
        Dim wc As New WebClient
        Dim reader As New IO.StreamReader(wc.OpenRead(URL))
        Return reader.ReadToEnd
    End Function

    <DnsPermission( _
        Security.Permissions.SecurityAction.Demand)> _
    Public Function MyIPAddresses() As String()
        Dim host As IPHostEntry = _
            Dns.GetHostByName(Dns.GetHostName())

        Dim ipList As IPAddress()
        ipList = host.AddressList

        Dim output As New ArrayList
        For Each ip As IPAddress In ipList
            output.Add(ip.ToString)
        Next
        Return output.ToArray(GetType(String))
    End Function

    <WebPermission( _
        Security.Permissions.SecurityAction.Demand)> _
    Private Sub CheckURL(ByVal url As String)
        Dim e As New WatchedURLEventArgs
        e.URLBeingWatched = url
        Me.m_lastCheck = Now
        e.CheckedTime = Me.m_lastCheck

        Try
            Dim wr As WebRequest = WebRequest.Create(url)
            If TypeOf wr Is HttpWebRequest Then
                Dim httpWR As HttpWebRequest
                httpWR = DirectCast(wr, HttpWebRequest)
                httpWR.Method = "HEAD"
                httpWR.AllowAutoRedirect = False
                Dim wResponse As WebResponse
                wResponse = httpWR.GetResponse()
                If TypeOf wResponse Is HttpWebResponse Then
                    Dim httpResponse As HttpWebResponse
                    httpResponse = DirectCast( _
                        wResponse, HttpWebResponse)
                    Dim response As Integer
                    response = CInt(httpResponse.StatusCode)
                    Me.m_lastResponse = _
                        httpResponse.StatusDescription

                    e.Status = response
                    e.StatusDescription = Me.lastResponse

                    If response >= 200 And response < 300 Then
                        e.URLAvailable = True
                        Me.m_WatchedURLAvailable = True
                    Else
                        Me.m_WatchedURLAvailable = False
                        e.URLAvailable = False
                    End If
                Else
                    Me.m_WatchedURLAvailable = False
                    Me.m_lastResponse = "Check Failed"

                    e.Status = 0
                    e.StatusDescription = Me.lastResponse
                End If
            Else
                Me.m_WatchedURLAvailable = False
                Me.m_lastResponse = "Invalid URL Type"

                e.Status = 0
                e.StatusDescription = Me.lastResponse
            End If
        Catch ex As Exception
            Me.m_WatchedURLAvailable = False
            Me.m_lastResponse = _
                "Error Checking: " & ex.Message

            e.Status = 0
            e.StatusDescription = Me.lastResponse
        Finally
            Me.OnWatchedURLChecked(e)
            RaiseEvent WatchedURLChecked(Me, e)
            If Not Me.WatchedURLAvailable Then
                Me.OnWatchedURLUnavailable(e)
                RaiseEvent WatchedURLUnavailable(Me, e)
            End If
        End Try
    End Sub

#Region "Events and Overridable Subs"

    Public Event WatchedURLChecked( _
        ByVal sender As Object, _
        ByVal e As WatchedURLEventArgs)

    Public Event WatchedURLUnavailable( _
        ByVal sender As Object, _
        ByVal e As WatchedURLEventArgs)

    Protected Overridable Sub OnWatchedURLChecked( _
        ByVal e As WatchedURLEventArgs)

    End Sub

    Protected Overridable Sub OnWatchedURLUnavailable( _
        ByVal e As WatchedURLEventArgs)

    End Sub
#End Region

#Region "Timer Event"

    Private Sub m_internalTimer_Elapsed( _
        ByVal sender As Object, _
        ByVal e As System.Timers.ElapsedEventArgs) _
        Handles m_internalTimer.Elapsed
        Me.m_internalTimer.Stop()
        If Now.Subtract(Me.m_lastCheck).TotalSeconds _
                >= Me.m_SecondsBetweenCheck Then
            If (Not Me.m_WatchedURL Is Nothing) _
                AndAlso (Not Me.m_WatchedURL = String.Empty) Then
                CheckURL(Me.m_WatchedURL)
            End If
        End If
        Me.m_internalTimer.Start()
    End Sub


#End Region

#Region "Property Procedures"

    Public Property WatchedURL() As String
        Get
            Return m_WatchedURL
        End Get
        Set(ByVal Value As String)
            m_WatchedURL = Value
        End Set
    End Property

    Public Property SecondsBetweenCheck() As Integer
        Get
            Return m_SecondsBetweenCheck
        End Get
        Set(ByVal Value As Integer)
            m_SecondsBetweenCheck = Value
        End Set
    End Property

    Public ReadOnly Property lastCheck() As DateTime
        Get
            Return m_lastCheck
        End Get
    End Property

    Public ReadOnly Property lastResponse() As String
        Get
            Return m_lastResponse
        End Get
    End Property

    Public ReadOnly Property WatchedURLAvailable() As Boolean
        Get
            Return m_WatchedURLAvailable
        End Get
    End Property


#End Region

End Class




